
## R version 3.3.0 ##
#install.packages("devtools")
#install.packages("dplyr")
#install.packages("scales")
#install.packages("tibble")
 
#install.packages("changepoint")
#install.packages("accelerometry")
#install.packages("installr")
#install.packages("flsa")

library(installr)
install.Rtools()
library(devtools)
install_github("twitter/BreakoutDetection", force=TRUE)
install_github("masalmon/convertagd", force=TRUE)

library(dplyr)
library(convertagd)
library(flsa)
library(stringr)
library(lubridate) ## for time overlapping "interval" function ##
library(BreakoutDetection)
library(changepoint)
library(accelerometry)


read_agd = function (file, tz = "GMT", sec=10) 
{
    con <- DBI::dbConnect(RSQLite::SQLite(), file)
    ad_set <- DBI::dbReadTable(con, "settings")
    ad_set <<- ad_set
    raw_start <- ad_set$settingValue[ad_set$settingName == "startdatetime"]
    origen <- as.POSIXlt((as.numeric(raw_start)/1e+07), origin = "0001-01-01 00:00:00", 
        tz = "GMT")
    origen <- as.POSIXct(as.character(origen), tz = tz)
    longitud <- ad_set$settingValue[ad_set$settingName == "epochcount"]
    longitud <- as.numeric(longitud)
    base <- DBI::dbReadTable(con, "data")
    base <- dplyr::tbl_df(base)
    base <- dplyr::mutate_(base, date = lazyeval::interp(~origen + 
        seq(from = 0, by = sec, length.out = longitud)))
    base <- dplyr::select_(base, .dots = list(quote(date), lazyeval::interp(~everything()), 
        quote(-dataTimestamp)))
    ad_set <- dplyr::tbl_df(ad_set)
    res <- list(settings = ad_set, raw.data = base)
    return(res)
}



toChar = function(num){
	char_num = str_pad(num, 4, pad = "0")
	return(char_num)
}

aggregate_dat=function(datain, cutnum=1, unit=c("min", "sec"), tz="GMT")
{
	dat = aggregate(. ~ cut(datain$date, paste(cutnum, unit)), datain[setdiff(names(datain), "date")], sum)
	names(dat)[1] = "date";
	dat$date = strptime(dat$date, "%Y-%m-%d %H:%M:%S", tz=tz)
	return(dat)
}

RemoveAfter7days = function(datain, a_day = 60*60*24)
{
	datain$date_only = lubridate::date(datain$date)
	n = nrow(datain)
	n_days = length(table(datain$date_only))
	## indexing wear-day ##
	intiation_time = datain$date[1]
	
	datain$wear_day = 0
	day1_idx = which(datain$date < intiation_time + a_day)
	n_day1 = length(day1_idx)
	ss = 1; ee=n_day1
	for(d in 1:(n_days-1)){
		idx = seq(from=ss, to=ee)
		datain$wear_day[idx] = d 
		ss = ss + n_day1
		ee = ifelse(ee+n_day1>n, n, ee+n_day1)
	}

	dataout = subset(datain, wear_day !=0) ## to remove data after 7days ##
	return(dataout)
}


ChangeTimeVar = function(datain, col_idx){
				dataout = datain
				for(i in col_idx) dataout[,i] = as.POSIXct(datain[,i], tz = "GMT", format="%Y-%m-%d %H:%M:%S")
				return(dataout)
}


GetDiary_subj = function(datain, subj_idx, idx_bed, idx_wake, tz="GMT"){
	################################### find child's diary info ###################################
	diary_bed = rep(as.POSIXct(datain[subj_idx, idx_bed[1]], tz = tz), 7)
	diary_wake = rep(as.POSIXct(datain[subj_idx, idx_wake[1]], tz = tz), 7)

	for(d in 1:7){
		diary_bed[d] = as.POSIXct(datain[subj_idx, idx_bed[d]], tz = tz)
		diary_wake[d] = as.POSIXct(datain[subj_idx,idx_wake[d]], tz = tz)
	}
	
	diary_time = data.frame(bed=diary_bed,wake=diary_wake)
	return(diary_time)
}

GetEstSleepInterval_subj = function(datain, bed="22:00:00", wake="8:00:00", tz="GMT"){
	################################### find child's diary info ###################################
	date_names = names(table(lubridate::date(datain$date)))
	n_date = length(date_names)
	bed_char = paste(date_names[-n_date], bed)
	wake_char = paste(date_names[-1], wake)
	
	bed_time = strptime(bed_char, "%Y-%m-%d %H:%M:%S", tz=tz)
	wake_time = strptime(wake_char, "%Y-%m-%d %H:%M:%S", tz=tz)

	
	sleep_interval = data.frame(bed=bed_time,wake=wake_time)
	return(sleep_interval)
}


cal_mean_idx = function(i, x, idx_start, idx_end) mean(x[idx_start[i]:idx_end[i]])
	
Cal_EmpMean = function(date, Y, fit, NoPA_cut = 0.7, sleep_mins = 30)
{
	beta_hat = round(fit,5)
	cp = rle(beta_hat)
	n_cp =length(cp$lengths)
	idx_end = cumsum(cp$lengths)
	idx_start = c(1,(idx_end +1)[-n_cp])

	time_intervals = difftime(date[idx_end], date[idx_start],  unit="mins")
	betas = beta_hat[idx_start]
	
	Y_k = sapply(X = 1:n_cp, cal_mean_idx, Y, idx_start, idx_end)
	PctZero = sapply(X = 1:n_cp, cal_mean_idx, I(Y==0), idx_start, idx_end)
	NoPA = round(PctZero >=NoPA_cut)
	EmpMean = rep(Y_k, cp$lengths)

	dat = data.frame(ts_start = date[idx_start], ts_end = date[idx_end], Intervals_mins = time_intervals, 
					 Mean=Y_k, betas = betas, PctZero = PctZero, NoPA=NoPA)
	
	cp_NoPA = rle(NoPA)
	n_cp_NoPA = length(cp_NoPA$lengths)
	idx_end2 = idx_end[cumsum(cp_NoPA$lengths)]
	idx_start2 = c(1,(idx_end2 +1)[-n_cp_NoPA])
	cp_NoPA_rep = idx_end2-idx_start2+1
	time_intervals2 = difftime(date[idx_end2], date[idx_start2],  unit="mins")

	Mean_NoPA = sapply(X = 1:n_cp_NoPA, cal_mean_idx, Y, idx_start2, idx_end2)
	PctZero_NoPA = sapply(X = 1:n_cp_NoPA, cal_mean_idx, I(Y==0), idx_start2, idx_end2)
	Sleep_NoPA = round(time_intervals2 >= sleep_mins )*round(PctZero_NoPA>=NoPA_cut)

	dat2 = data.frame(ts_start = date[idx_start2], ts_end = date[idx_end2], 
					  Intervals_mins = time_intervals2, 
					  Mean=Mean_NoPA, PctZero = PctZero_NoPA, NoPA_cut=NoPA_cut, 
					  Sleep=Sleep_NoPA, NoPA = round(PctZero_NoPA>=NoPA_cut))

	return(list(EmpMean=EmpMean, summary = dat, change_point = idx_end[-n_cp] , dat2=dat2))
}


MeanByCPS = function(Y, cps){
			n_seg = length(cps)+1
			idx_start = c(1, cps+1)
			idx_end = c(cps, length(Y))
			return(sapply(X = 1:n_seg, cal_mean_idx, Y, idx_start, idx_end))
}

ChoosingLambdaRange3 = function(Y, N_steps = 50){
	out=cpt.mean(Y, Q=N_steps, method="BinSeg", test.stat="Normal", param.estimates=FALSE)
	n_cps = rowSums((!is.na(cpts.full(out))))
	pen_values2 = pen.value.full(out) 
	min_lambda = pen_values2[N_steps]
	max_lambda = min_lambda*3
	out=cpt.mean(Y, pen.value=c(min_lambda, max_lambda), penalty="CROPS", method="PELT", test.stat="Normal")
	n_cps = rowSums((!is.na(cpts.full(out))))
	n_steps = n_cps + 1
	pen_values = pen.value.full(out) 

	idx = which.min(abs(N_steps - n_steps))
	return(list(lambda = pen_values[idx], N_steps = n_cps[idx]+1))
}


SleepEstEachDay = function(datain, Method="FLASSO", id = NA, 
					   Y_name = "max_count", Y_threshold = 5, 
					   nonwear_detect = FALSE, wear_mins = 120,
					   N_steps = 50, NoPA_cut = 0.7, sleep_mins = 20, wake_mins = 180,
					   nap_mins = 20, 
					   NoPA_cut_adapt = TRUE,
					   UseDiary = FALSE, diary_data = c(),
					   CommonBedTime = "22:00:00", CommonWakeTime = "8:00:00", tz = "GMT",
					   visual = TRUE)
{
	if(UseDiary == FALSE)
		SI_i = GetEstSleepInterval_subj(datain, bed=CommonBedTime, wake=CommonWakeTime, tz=tz)
	if(UseDiary == TRUE) 
		SI_i = diary_data

	d=1
	idx_Y = which(names(datain) == Y_name)
	Y = datain[,idx_Y]
	
	###### wear_time validation ######
	valid_accel_data = 1;
	if(nonwear_detect ==TRUE){
		epoch_secs = difftime(datain$date[2], datain$date[1], unit="secs")
		if(epoch_secs < 60) dat_min = aggregate_dat(datain, cutnum=1, unit="min")
		if(epoch_secs == 60) dat_min = datain
		N_min = nrow(dat_min)
		N_wear = sum(accel.weartime(dat_min[,idx_Y], window = wear_mins))
		if(N_min != N_wear) valid_accel_data = 0
	}
	###### wear_time validation end ######
	
	Y[which(Y <=Y_threshold)] = 0
	N = length(Y)
	
	if(valid_accel_data){	
		n_overlap = 0
		#### part for threshold adaptation ####
		while(n_overlap == 0 & NoPA_cut > 0){
			if(Method=="PELT"){
				capture.output(lambda <- ChoosingLambdaRange3(Y, N_steps)$lambda, file='NUL')
				capture.output(out <- cpt.mean(Y, pen.value=lambda, penalty="Manual", method="PELT", test.stat="Normal"), file='NUL')
				cps = cpts(out)
				mean_Y = rep(coef(out)$mean, diff(c(0, cps, N)))
				xx = Cal_EmpMean(datain$date, Y, mean_Y, NoPA_cut = NoPA_cut, sleep_mins = sleep_mins)
				df = length(cps)+1
			}
			
			if(Method=="FLASSO"){
				out = flsaTopDown(Y, lambda1=0, groups=N_steps, lambda2=NULL)
				xx = Cal_EmpMean(datain$date, Y, out$Solution[1,], NoPA_cut = NoPA_cut, sleep_mins = sleep_mins)	
				df = N_steps
			}
			
			#### find time intervals which are overlapping w/ an external time interval
			int2 = interval(SI_i[d,1], SI_i[d,2], tzone="GMT")
			xx$dat2$overlap = 0
			for(j in 1:nrow(xx$dat2)){
				int1 = interval(xx$dat2[j,1], xx$dat2[j,2], tzone="GMT")
				xx$dat2$overlap[j] = intersect(int1, int2)
			}
			xx$dat2$overlap[!is.na(xx$dat2$overlap)] = 1
			xx$dat2$overlap[is.na(xx$dat2$overlap)] = 0
			
			idx_sleep = which(xx$dat2$overlap == 1 & xx$dat2$Sleep ==1)
			idx_NoPA = which(xx$dat2$overlap == 0 & xx$dat2$NoPA == 1 & xx$dat2$Intervals_mins > nap_mins)
			
			## number of overlap between external info and estimated sleep intervals ##
			n_overlap = length(idx_sleep)
			
			## number of segments for non-physical activity ##
			n_NoPA = length(idx_NoPA)
			
			if(NoPA_cut_adapt==FALSE) break
			NoPA_cut = NoPA_cut - 0.05
		}
		
		if(n_NoPA==0){
			tmp_sleep = data.frame(ts_start = as.POSIXct(NA, tz = "GMT", origin="1970-01-01"), 
								 ts_end =as.POSIXct(NA, tz = "GMT", origin="1970-01-01"), 
								 Intervals_mins = 0, Mean = NA, PctZero = NA, NoPA_cut = NoPA_cut, 
								 Sleep = 0, NoPA = 0, overlap = 0)	
								 
			NoPA_tab = data.frame(id = id, valid_accel = valid_accel_data, df=df,
								  wear_day=d, days = wday(datain$date[1], label=TRUE),
								  tmp_sleep, n_NoPA = 0)									 
		}else{
			NoPA_tab = data.frame(id = id, valid_accel = valid_accel_data, df=df,
								  wear_day=d, days = wday(datain$date[1], label=TRUE),
								  xx$dat2[idx_NoPA,], n_NoPA = n_NoPA)
		}
		
		if(n_overlap==0){
			tmp_sleep = data.frame(ts_start = as.POSIXct(NA, tz = "GMT", origin="1970-01-01"), 
								 ts_end =as.POSIXct(NA, tz = "GMT", origin="1970-01-01"), 
								 Intervals_mins = NA, Mean = NA, PctZero = NA, NoPA_cut = NoPA_cut, 
								 Sleep = NA, NoPA = NA, overlap = NA)			
		
			sleep_tab = data.frame(id = id, valid_accel = valid_accel_data, df=df,
					  wear_day=d, days = wday(datain$date[1], label=TRUE),
					  tmp_sleep,
					  SI_bed = SI_i$bed[d], SI_wake = SI_i$wake[d], n_sleep = n_overlap)
		}else{
			sleep_tab = data.frame(id = id, valid_accel = valid_accel_data, df=df,
									  wear_day=d, days = wday(datain$date[1], label=TRUE),
									  xx$dat2[idx_sleep,],
									  SI_bed = SI_i$bed[d], SI_wake = SI_i$wake[d], n_sleep = n_overlap)
		}
			
	}else{
		tmp_sleep = data.frame(ts_start = as.POSIXct(NA, tz = "GMT", origin="1970-01-01"), 
							 ts_end =as.POSIXct(NA, tz = "GMT", origin="1970-01-01"), 
							 Intervals_mins = NA, Mean = NA, PctZero = NA, NoPA_cut = NA, 
							 Sleep = NA, NoPA = NA, overlap = NA)

		sleep_tab = data.frame(id = id, valid_accel = valid_accel_data, df=NA, 
								  wear_day=d, days = wday(datain$date[1], label=TRUE), 
								  tmp_sleep,
								  SI_bed = SI_i$bed[d], SI_wake = SI_i$wake[d], n_sleep = NA)	
								  
		NoPA_tab = data.frame(id = id, valid_accel = valid_accel_data, df=NA,
				  wear_day=d, days = wday(datain$date[1], label=TRUE),
				  tmp_sleep, n_NoPA = NA)
		
	}
	
	sleep_tab$ts_start = as.character(sleep_tab$ts_start)
	sleep_tab$ts_end = as.character(sleep_tab$ts_end)
	NoPA_tab$ts_start = as.character(NoPA_tab$ts_start)
	NoPA_tab$ts_end = as.character(NoPA_tab$ts_end)		
	
	summary_sleep = sleep_tab
	summary_NoPA = NoPA_tab
	
	Dat_list = NULL
	if(visual == TRUE){
		if(valid_accel_data){
			EmpMean=xx$EmpMean
		}else{
			EmpMean=rep(0, length(Y))
		}
		Dat_list[[d]] = data.frame(date = datain$date, Y = Y, EmpMean = EmpMean)
	}

	idx = which(nchar(summary_sleep$ts_start) == 10)
	summary_sleep$ts_start[idx] = paste(summary_sleep$ts_start[idx], "00:00:00")
	idx = which(nchar(summary_sleep$ts_end) == 10)
	summary_sleep$ts_end[idx] = paste(summary_sleep$ts_end[idx], "00:00:00")

	idx = which(nchar(summary_NoPA$ts_start) == 10)
	summary_NoPA$ts_start[idx] = paste(summary_NoPA$ts_start[idx], "00:00:00")
	idx = which(nchar(summary_NoPA$ts_end) == 10)
	summary_NoPA$ts_end[idx] = paste(summary_NoPA$ts_end[idx], "00:00:00")
	
	return(list(summary_sleep=summary_sleep, summary_NoPA = summary_NoPA, dat = Dat_list, visual = visual))
}
